@@ -1,5 +1,5 @@
 package Photonic::LE::NR2::SHChiTensor;
-$Photonic::LE::NR2::SHChiTensor::VERSION = '0.016';
+$Photonic::LE::NR2::SHChiTensor::VERSION = '0.017';
 
 =encoding UTF-8
 
@@ -9,7 +9,7 @@ Photonic::LE::NR2::SHChiTensor
 
 =head1 VERSION
 
-version 0.016
+version 0.017
 
 =head1 COPYRIGHT NOTICE
 
@@ -164,9 +164,7 @@ Spectral variables
 use namespace::autoclean;
 use PDL::Lite;
 use PDL::NiceSlice;
-use PDL::Complex;
 use PDL::MatrixOps;
-use Storable qw(dclone);
 use PDL::IO::Storable;
 use Photonic::Utils qw(make_haydock tensor);
 use Photonic::Types;
@@ -259,13 +257,12 @@ sub evaluate {
 	my $nrsh=Photonic::LE::NR2::SH->new(
 	    shp=>$_, epsA1=>$epsA1, epsB1=>$epsB1, epsA2=>$epsA2,
 	    epsB2=>$epsB2, filterflag=>0);
-	# RorI, XorY,nx,ny
+	# XorY,nx,ny
 	# dipolar, quadrupolar, external, full
 	my $P2 = $nrsh->$method;
-	my $P2M=$P2->mv(0,-1)->mv(0,-1)
-	    ->clump(-3) #linear index, RorI, XorY
-	    ->mv(-2,0) #RorI, index, XorY
-	    ->sumover  #RorI, XorY
+	my $P2M=$P2->mv(0,-1)
+	    ->clump(-2) #linear index, XorY
+	    ->sumover  #XorY
 	    /$self->geometry->npoints;
 	my $k=$_->nrf->nr->geometry->Direction0;
 	my $FPChi=$epsT-identity($nd); #four pi chi linear 2w
@@ -278,10 +275,9 @@ sub evaluate {
 	    $f=$mask->sum/$self->geometry->npoints; #filling fraction
 				#of mask
 	    $P2*=$mask->(*1); #masked polarization
-	    $P2Mmask=$P2->mv(0,-1)->mv(0,-1) #masked macroscopic polarization
-	    ->clump(-3) #linear index, RorI, XorY
-	    ->mv(-2,0) #RorI, index, XorY
-	    ->sumover  #RorI, XorY
+	    $P2Mmask=$P2->mv(0,-1) #masked macroscopic polarization
+	    ->clump(-2) #linear index, XorY
+	    ->sumover  #XorY
 		/$self->geometry->npoints;
 	}
 	$P2Mmask += $f*$Dep2 if $KIND2SUBTRACT{$kind}; # subtract masked macro depolarization field
@@ -291,10 +287,10 @@ sub evaluate {
     #I have to convert from the array of polarizations for given
     #directions to the actual cartesian chi's.
     #$P2Mp has cartesian, dyad indices
-    my $P2Mp = PDL->pdl(@P2M)->complex;
+    my $P2Mp = PDL->pdl(@P2M);
     #Get cartesian indices out of the way, solve the system of
     #equations, and move the cartesian indices back
-    my $chiTensor=tensor($P2Mp->mv(1,-1), $self->geometry->unitDyadsLU, $nd, 3, sub { $_[0]->mv(-1,1) });
+    my $chiTensor=tensor($P2Mp->mv(0,-1), $self->geometry->unitDyadsLU, $nd, 3, sub { $_[0]->mv(-1,0) });
     $self->_chiTensor($chiTensor);
     return $chiTensor;
 }
@@ -303,10 +299,10 @@ sub evaluate {
 
 sub _build_nrshp { # One Haydock coefficients calculator per direction0
     my $self=shift;
-    my $nr = make_haydock($self, 'Photonic::LE::NR2::AllH', 1);
-    my ($i, @nrshp) = 0;
-    foreach(@{$self->geometry->unitPairs}){
-	my @args=(nr=>$nr->[$i++], nh=>$self->nhf, smallE=>$self->smallE);
+    my $nr = make_haydock($self, 'Photonic::LE::NR2::AllH', $self->geometry->unitPairs, 1, qw(reorthogonalize use_mask mask));
+    my @nrshp;
+    foreach(@$nr){
+	my @args=(nr=>$_, nh=>$self->nhf, smallE=>$self->smallE);
 	push @args, filter=>$self->filter if $self->has_filter;
 	my $nrf=Photonic::LE::NR2::Field->new(@args);
 	my $nrshp=Photonic::LE::NR2::SHP->
