@@ -1,5 +1,5 @@
 package Photonic::LE::S::EpsL;
-$Photonic::LE::S::EpsL::VERSION = '0.016';
+$Photonic::LE::S::EpsL::VERSION = '0.017';
 
 =encoding UTF-8
 
@@ -9,7 +9,7 @@ Photonic::LE::S::EpsL
 
 =head1 VERSION
 
-version 0.016
+version 0.017
 
 =head1 COPYRIGHT NOTICE
 
@@ -112,7 +112,6 @@ check.
 use namespace::autoclean;
 use PDL::Lite;
 use PDL::NiceSlice;
-use PDL::Complex;
 use Photonic::LE::S::AllH;
 use Photonic::Types;
 use Photonic::Utils qw(lentzCF);
@@ -130,8 +129,8 @@ with 'Photonic::Roles::EpsL';
 
 after BUILD => sub {
     my $self=shift;
-    my $as=pdl(map r2C($_), @{$self->nr->as})->cplx;
-    my $b2s=pdl(map r2C($_), @{$self->nr->b2s})->cplx;
+    my $as=$self->nr->as;
+    my $b2s=$self->nr->b2s;
     my $min= min($self->nh, $self->nr->iteration);
     my ($fn, $n)=lentzCF($as, -$b2s, $min, $self->smallE);
     # Check this logic:
