@@ -1,5 +1,5 @@
 package Photonic::Roles::ReorthogonalizeR;
-$Photonic::Roles::ReorthogonalizeR::VERSION = '0.016';
+$Photonic::Roles::ReorthogonalizeR::VERSION = '0.017';
 
 =encoding UTF-8
 
@@ -9,7 +9,7 @@ Photonic::Roles::ReorthogonalizeC
 
 =head1 VERSION
 
-version 0.016
+version 0.017
 
 =head1 COPYRIGHT NOTICE
 
@@ -51,9 +51,9 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA  02110-1301 USA
 =head1 DESCRIPTION
 
 Roles consumed by AllH objects to be used in a Photonic
-calculation. Estimates orthogonlization errors and makes a full
-reorthogoalization of Haydock states when required. This version works
-for Hermitian operators with real Hayock coefficients.
+calculation. Estimates orthogonalization errors and makes a full
+reorthogonalization of Haydock states when required. This version works
+for Hermitian operators with real Haydock coefficients.
 
 =head1 ATTRIBUTES
 
@@ -182,9 +182,9 @@ sub _checkorthogonalize {
     return unless $self->reorthogonalize;
     return if $self->fullorthogonalize_N; #already orthogonalizing
     my $n=$self->iteration;
-    my $a=PDL->pdl($self->as);
-    my $b=PDL->pdl($self->bs);
-    my $c=PDL->pdl($self->cs);
+    my $a=$self->as;
+    my $b=$self->bs;
+    my $c=$self->cs;
     if($self->_justorthogonalized){
 	$self->_write_justorthogonalized(0);
 	my $current_W=PDL->ones($n)*$self->noise;
