@@ -1,5 +1,5 @@
 package Photonic::WE::R2::OneH;
-$Photonic::WE::R2::OneH::VERSION = '0.016';
+$Photonic::WE::R2::OneH::VERSION = '0.017';
 
 =encoding UTF-8
 
@@ -37,12 +37,10 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA  02110-1301 USA
 use namespace::autoclean;
 use PDL::Lite;
 use PDL::NiceSlice;
-use PDL::FFTW3;
-use PDL::Complex;
 use Carp;
 use Moose;
 use MooseX::StrictConstructor;
-use Photonic::Utils qw(MHProd any_complex);
+use Photonic::Utils qw(MHProd any_complex GtoR RtoG);
 use Photonic::Types;
 
 has 'metric'=>(is=>'ro', isa => 'Photonic::WE::R2::Metric',
@@ -61,17 +59,17 @@ sub applyOperator {
     my $mask=undef;
     $mask=$self->mask if $self->use_mask;
     my $gpsi=$self->applyMetric($psi);
-    # gpsi is RorI xyz nx ny nz. Get cartesian out of the way and
-    # transform to real space. Note FFFTW3 wants real PDL's[2,...]
-    my $gpsi_r=ifftn($gpsi->mv(1,-1), $self->ndims);
-    #$psi_r is RorI nx ny nz  xyz, B is nx ny nz
+    # gpsi is xyz nx ny nz. Get cartesian out of the way and
+    # transform to real space. Note FFTW3 wants real PDL's[2,...]
+    my $gpsi_r=GtoR($gpsi, $self->ndims, 1)->mv(0,-1);
+    #$psi_r is nx ny nz  xyz, B is nx ny nz
     # Multiply by characteristic function
     my $Bgpsi_r=$gpsi_r * $self->B->r2C;
-    #Bpsi_r is RorI nx ny nz  xyz
+    #Bpsi_r is nx ny nz  xyz
     #Transform to reciprocal space, move xyz back and make complex,
-    my $psi_G=fftn($Bgpsi_r, $self->ndims)->mv(-1,1);
+    my $psi_G=RtoG($Bgpsi_r->mv(-1,0), $self->ndims, 1);
     #Apply mask
-    #psi_G is ri:xy:nx:ny mask is nx:ny
+    #psi_G is xy:nx:ny mask is nx:ny
     $psi_G=$psi_G*$mask->(*1) if defined $mask; #use dummy for xy
     return $psi_G;
 }
@@ -79,11 +77,11 @@ sub applyOperator {
 sub applyMetric {
     my $self=shift;
     my $psi=shift;
-    #psi is RorI xy.. nx ny..
+    #psi is xy.. nx ny..
     my $g=$self->metric->value;
     #$g is xyz xyz nx ny nz
-    my $gpsi=($g*$psi(:,:,*1))->sumover; #matrix times vector
-    #$gpsi is RorI xy.. nx ny..
+    my $gpsi=($g*$psi(:,*1))->sumover; #matrix times vector
+    #$gpsi is xy.. nx ny..
     return $gpsi;
 }
 
@@ -111,13 +109,12 @@ sub _firstState {
     my $v=PDL->zeroes(@{$self->dims}); #build a nx ny nz pdl
     my $arg=join ',', ("(0)") x $d; #(0),(0),... ndims times
     $v->slice($arg).=1; #delta_{G0}
-    my $e=$self->polarization; #RorI xyz
-    croak "Polarization has wrong dimensions. " .
-	  " Should be $d-dimensional complex vector."
-	unless any_complex($e) && $e->ndims==2 &&
-	$e->dim(0)==2 && $e->dim(1)==$d;
-    my $modulus2=$e->Cabs2->sumover;
-    croak "Polarization should be non null" unless
+    my $e=$self->polarization; #xyz
+    confess "Polarization has wrong dimensions. " .
+	  " Should be $d-dimensional complex vector, got ($e)."
+	unless any_complex($e) && $e->dim(0)==$d;
+    my $modulus2=$e->abs2->sumover;
+    confess "Polarization should be non null" unless
 	$modulus2 > 0;
     $e=$e/sqrt($modulus2);
     $self->_normalizedPolarization($e);
@@ -126,7 +123,6 @@ sub _firstState {
     return $phi;
 }
 
-
 __PACKAGE__->meta->make_immutable;
 
 1;
@@ -137,7 +133,7 @@ Photonic::OneH::R2
 
 =head1 VERSION
 
-version 0.016
+version 0.017
 
 =head1 SYNOPSIS
 
@@ -177,7 +173,7 @@ A Photonic::Metric::R2 object defining the geometry of the
 system, the charateristic function, the wavenumber, wavevector and
 host dielectric function. Required in the initializer.
 
-=item * polarization PDL::Complex
+=item * polarization
 
 A non null vector defining the complex direction of the macroscopic
 field.
