@@ -1,5 +1,5 @@
 package Photonic::WE::S::GreenP;
-$Photonic::WE::S::GreenP::VERSION = '0.016';
+$Photonic::WE::S::GreenP::VERSION = '0.017';
 
 =encoding UTF-8
 
@@ -9,7 +9,7 @@ Photonic::WE::S::GreenP
 
 =head1 VERSION
 
-version 0.016
+version 0.017
 
 =head1 COPYRIGHT NOTICE
 
@@ -119,7 +119,6 @@ Criteria of convergence. 0 means don't check.
 
 use namespace::autoclean;
 use PDL::Lite;
-use PDL::Complex;
 use Photonic::WE::S::AllH;
 use Photonic::Types;
 use Photonic::Utils qw(lentzCF);
@@ -145,8 +144,8 @@ sub _build_Gpp {
     my $self=shift;
     $self->haydock->run unless $self->haydock->iteration;
     my $epsR=$self->haydock->epsilonR;
-    my $as=pdl(map r2C($_), @{$self->haydock->as})->cplx;
-    my $bcs=pdl(map r2C($_), @{$self->haydock->bcs})->cplx;
+    my $as=$self->haydock->as;
+    my $bcs=$self->haydock->bcs;
     my $min= min($self->nh, $self->haydock->iteration);
     #    b0+a1/b1+a2/...
     #	lo debo convertir a
@@ -157,7 +156,7 @@ sub _build_Gpp {
     #of them were used, there is no remaining work to do, so, converged
     $self->_converged($n<$min || $self->haydock->iteration<=$self->nh);
     $self->_nhActual($n);
-    my $g0b02=$self->haydock->gs->[0]*$self->haydock->b2s->[0];
+    my $g0b02=$self->haydock->gs->[0]*$self->haydock->b2s->slice("(0)");
     return $g0b02/($epsR*$fn);
 }
 
